@@ -6,8 +6,11 @@ test -r ChangeLog || touch -t 198001010000 ChangeLog
 MYDIR=`dirname $0`
 if test -r batch/make/version; then
     echo "Generating version..."
-    echo "m4_define(AUTOMATIC_VERSION,["`sh batch/make/version $MYDIR`"])" > version || exit 1
+    echo "m4_define(AUTOMATIC_VERSION,["`sh batch/make/version $MYDIR`"])" > version.m4 || exit 1
+    rm -f version
 fi
+echo "Copying license..."
+test -r COPYING || cp COPYING.txt COPYING || exit $?
 echo "Running aclocal..."
 aclocal || { rm aclocal.m4; exit 1; }
 echo "Running autoheader..."
