@@ -3,6 +3,4 @@
 # determines the relevant related branch for git submodules
 
 wdd=`dirname $0`
-MAIN_BRANCH=legacy_0.2.8.3
-UBUNTU_BRANCH=ubuntu-bionic-0.2.8.3
-
+MAIN_BRANCH=legacy_0.2.8
