@@ -168,8 +168,8 @@ static nTimeRolling se_CalcTimeout( double score )
     return se_CalcScore2( score ) * se_prefixSpamTimeoutMultiplier;
 }
 
-eChatSaidEntry::eChatSaidEntry(const tString & said, const nTimeRolling & t, eChatMessageType type)
-: said_( said ), time_( t ), type_( type )
+eChatSaidEntry::eChatSaidEntry(const tString & said, const tString & name, const nTimeRolling & t, eChatMessageType type)
+: said_( said ), playerName_(name), time_( t ), type_( type )
 {
 }
 
@@ -182,6 +182,11 @@ const tString & eChatSaidEntry::Said() const
     return said_;
 }
 
+const tString & eChatSaidEntry::PlayerName() const
+{
+    return playerName_;
+}
+
 const nTimeRolling & eChatSaidEntry::Time() const
 {
     return time_;
@@ -215,7 +220,7 @@ bool eChatLastSaid::Prefix::StartsWith( const eChatLastSaid::Prefix & other ) co
 
 
 eChatLastSaid::eChatLastSaid()
-: lastSaid_(), knownPrefixes_()
+  : lastSaid_(), knownPrefixes_(), disconnected_(false)
 {
 }
 
@@ -233,8 +238,20 @@ const eChatLastSaid::PrefixList & eChatLastSaid::KnownPrefixes() const
     return knownPrefixes_;
 }
 
+bool eChatLastSaid::Disconnected() const
+{
+    return disconnected_;
+}
+
+void eChatLastSaid::MarkDisconnected()
+{
+    disconnected_ = true;
+}
+
 void eChatLastSaid::AddSaid( const eChatSaidEntry & saidEntry )
 {
+    disconnected_ = false;
+
     if ( lastSaid_.size() >= static_cast< size_t >( se_lastSaidMaxEntries ) )
         lastSaid_.pop_back();
     
@@ -243,6 +260,7 @@ void eChatLastSaid::AddSaid( const eChatSaidEntry & saidEntry )
 
 nTimeRolling eChatLastSaid::AddPrefix( const tString & s, REAL score, nTimeRolling now )
 {
+    disconnected_ = false;
     nTimeRolling timeoutAt = now + se_CalcTimeout( score );
     Prefix prefix( s, score, timeoutAt );
     knownPrefixes_.push_back( prefix );
@@ -272,7 +290,7 @@ bool eChatSpamTester::Check()
     nTimeRolling currentTime = tSysTimeFloat();
     
     // check if the player already said the same thing not too long ago
-    eChatLastSaid::SaidList const & lastSaid = player_->lastSaid_.LastSaid();
+    eChatLastSaid::SaidList const & lastSaid = player_->GetLastSaid().LastSaid();
     const size_t saidSize = lastSaid.size();
     for ( size_t i = 0; i < saidSize; i++ )
     {
@@ -284,7 +302,7 @@ bool eChatSpamTester::Check()
         }
     }
     
-    eChatSaidEntry saidEntry( say_, currentTime, lastSaidType_ );
+    eChatSaidEntry saidEntry( say_, player_->GetUserName(), currentTime, lastSaidType_ );
     
     // check for prefix spam
     if ( se_prefixSpamShouldEnable )
@@ -335,14 +353,14 @@ bool eChatSpamTester::Check()
     }
 #endif
     
-    player_->lastSaid_.AddSaid( saidEntry );
+    player_->GetLastSaid().AddSaid( saidEntry );
     
     return false;
 }
 
 bool eChatSpamTester::CheckSpam( REAL factor, tOutput const & message ) const
 {
-    if ( nSpamProtection::Level_Mild <= player_->chatSpam_.CheckSpam( factor, player_->Owner(), message ) )
+    if ( nSpamProtection::Level_Mild <= player_->GetChatSpam().CheckSpam( factor, player_->Owner(), message ) )
         return true;
 
     return false;
@@ -390,7 +408,7 @@ bool eChatPrefixSpamTester::Check( tString & out, nTimeRolling & timeOut, eChatP
     }
         
     
-    eChatLastSaid::SaidList & lastSaid = player_->lastSaid_.lastSaid_;
+    eChatLastSaid::SaidList & lastSaid = player_->GetLastSaid().lastSaid_;
     
     // Map of Prefix => Data
     std::map< tString, PrefixEntry > foundPrefixes;
@@ -434,7 +452,7 @@ bool eChatPrefixSpamTester::Check( tString & out, nTimeRolling & timeOut, eChatP
 #ifdef DEBUG
                 con << "Spam prefix found: \"" << se_EscapeColors( prefix ) << "\" with score " << data.score << '\n';
 #endif
-                nTimeRolling t = player_->lastSaid_.AddPrefix( prefix, data.score, say_.Time() );
+                nTimeRolling t = player_->GetLastSaid().AddPrefix( prefix, data.score, say_.Time() );
                 timeOut = RemainingTime( t );
                 
                 // We caught the prefix. Don't catch words that start with the prefix.
@@ -460,7 +478,7 @@ void eChatPrefixSpamTester::CalcScore( PrefixEntry & data, const int & len, cons
     data.score += se_CalcScore2( se_CountColorCodes( prefix ) * se_prefixSpamNumberColorCodesMultiplier );
 
     // Apply based on number of known prefixes.
-    data.score += se_CalcScore2( player_->lastSaid_.KnownPrefixes().size() * se_prefixNumberKnownPrefixesMultiplier );
+    data.score += se_CalcScore2( player_->GetLastSaid().KnownPrefixes().size() * se_prefixNumberKnownPrefixesMultiplier );
 
     // Apply multiplier for annoying color messages
     if ( se_StartsWithColorCode( prefix ) )
@@ -469,14 +487,14 @@ void eChatPrefixSpamTester::CalcScore( PrefixEntry & data, const int & len, cons
 
 void eChatPrefixSpamTester::RemovePrefixEntries( const tString & prefix, const eChatSaidEntry & e ) const
 {
-    eChatSaidEntry entry( prefix, e.Time(), e.Type() );
-    eChatLastSaid::SaidList & xs = player_->lastSaid_.lastSaid_;
+    eChatSaidEntry entry( prefix, player_->GetUserName(), e.Time(), e.Type() );
+    eChatLastSaid::SaidList & xs = player_->GetLastSaid().lastSaid_;
     xs.erase( std::remove_if( xs.begin(), xs.end(), IsPrefixPredicate< eChatSaidEntry >( entry, false ) ), xs.end() );
 }
 
 bool eChatPrefixSpamTester::HasKnownPrefix( tString & out, nTimeRolling & timeOut ) const
 {
-    eChatLastSaid::PrefixList & prefixes = player_->lastSaid_.knownPrefixes_;
+    eChatLastSaid::PrefixList & prefixes = player_->GetLastSaid().knownPrefixes_;
     eChatLastSaid::Prefix testPrefix( say_.Said(), 0, 0 );
     
     eChatLastSaid::PrefixList::iterator it =
@@ -503,7 +521,7 @@ nTimeRolling eChatPrefixSpamTester::RemainingTime( nTimeRolling t ) const
 
 void eChatPrefixSpamTester::RemoveTimedOutPrefixes() const
 {
-    eChatLastSaid::PrefixList & xs = player_->lastSaid_.knownPrefixes_;
+    eChatLastSaid::PrefixList & xs = player_->GetLastSaid().knownPrefixes_;
     tString empty;
     eChatLastSaid::Prefix entry( empty, 0, say_.Time() );
     xs.erase( std::remove_if( xs.begin(), xs.end(), TimeOutPredicate< eChatLastSaid::Prefix >( entry ) ), xs.end() );
@@ -554,6 +572,19 @@ void eShuffleSpamTester::Reset()
     numberShuffles_ = 0;
 }
 
+tString eShuffleSpamTester::ShuffleMessage( ePlayerNetID *player, int position ) const
+{
+    tString message;
+    message << tOutput( "$team_preshuffle", player->GetColoredName() + "0xRESETT", position );
+    
+    if ( ShouldDisplaySuppressMessage() )
+        message << " " << tOutput( "$team_shuffle_suppress" );
+    else
+        message << '\n';
+    
+    return message;
+}
+
 tString eShuffleSpamTester::ShuffleMessage( ePlayerNetID *player, int fromPosition, int toPosition ) const
 {
     tString message;
