@@ -80,11 +80,14 @@ class eVoter;
 class ePlayer: public uPlayerPrototype{
     friend class eMenuItemChat;
     static uActionPlayer s_chat;
+    static uActionTooltip s_chatTooltip;
 
     tConfItemBase *configuration[PLAYER_CONFITEMS];
     int            CurrentConfitem;
     void   StoreConfitem(tConfItemBase *c);
     void   DeleteConfitems();
+
+    double lastTooltip_;
 public:
     tString    name;                 // the player's screen name
     tString    globalID;             // the global ID of the player in user@authority form
@@ -131,6 +134,9 @@ public:
 
     static bool PlayerIsInGame(int p);
 
+    // veto function for tooltips that require a controllable game object
+    static bool VetoActiveTooltip(int player);
+
     static rViewport * PlayerViewport(int p);
 
     static void LogIn();          //!< sends authentication login messages for all local players
@@ -153,6 +159,60 @@ private:
     tAccessLevel     accessLevel;    //!< admin access level of the current user
 };
 
+//! detector for turn timing assist bots
+class eUncannyTimingDetector
+{
+public:
+    //! settings for a single analyzer
+    struct eUncannyTimingSettings
+    {
+        REAL timescale; //!< the timescale. Events are divided in two buckets, one between 0 and timescale/2, the other from timescale/2 to timescale.
+        REAL maxGoodRatio; //!< the maximal allowed recent ratio of events to land in the 'good' bucket
+        REAL goodHumanRatio; //!< the maximal observed ratio for a human
+        int  averageOverEvents; //!< number of events to average over
+        
+        mutable REAL bestRatio; //!< best ratio achieved by players during this session
+
+        eUncannyTimingSettings( REAL ts, REAL human, REAL max )
+        : timescale( ts ), maxGoodRatio( max ), goodHumanRatio(human), averageOverEvents(40)
+        , bestRatio(0)
+        {}
+
+        ~eUncannyTimingSettings();
+    };
+
+    //! single analyzer with single timescale
+    class eUncannyTimingAnalysis
+    {
+    public:
+        //! analyze a single timing event
+        REAL Analyze( REAL timing, eUncannyTimingSettings const & settings );
+        eUncannyTimingAnalysis();
+    private:
+        REAL accurateRatio; //!< ratio of events in the more accurate half
+        int turnsSoFar;     //!< number of turns accounted for so far
+    };
+
+    //! detection level of timing aid hacks
+    enum DangerLevel
+    {
+        DangerLevel_Low,    //!< about 25% of the tolerance reached
+        DangerLevel_Medium, //!< about 50% of the tolerance reached
+        DangerLevel_High,   //!< about 75% of the tolerance reached
+        DangerLevel_Max     //!< 100% of the tolerance reached, worst action triggered
+    };
+
+    eUncannyTimingDetector();
+
+    //! analzye a timing event
+    void Analyze( REAL timing, ePlayerNetID * player );
+private:
+    //! three analyzers for varying timescales
+    eUncannyTimingAnalysis fast, medium, slow;
+
+    DangerLevel dangerLevel;
+};
+
 // the class that identifies players across the network
 class ePlayerNetID: public nNetObject, public eAccessLevelHolder{
     friend class ePlayer;
@@ -175,7 +235,6 @@ private:
     tCONTROLLED_PTR(eTeam)			nextTeam;		// the team we're in ( logically )
     tCONTROLLED_PTR(eTeam)			currentTeam;	// the team we currently are spawned for
     eTeamSet                        invitations_;   // teams this player is invited to
-    tCONTROLLED_PTR(eVoter)			voter_;			// voter assigned to this player
 
     tCHECKED_PTR(eNetGameObject) object; // the object this player is
     // controlling
@@ -201,6 +260,8 @@ private:
     //For improved remoteadmin
     tAccessLevel     lastAccessLevel;//!< access level at the time of the last name update
 
+    eUncannyTimingDetector uncannyTimingDetector_; //!< detector for timingbots
+
     nMachine *      registeredMachine_; //!< the machine the player is registered with
     void RegisterWithMachine();         //!< registers with a machine
     void UnregisterWithMachine();       //!< un registers with a machine
@@ -210,7 +271,7 @@ public:
         ChatFlags_Chat = 1,
         ChatFlags_Away = 2,
         ChatFlags_Menu = 4,
-        ChatFlags_Console = 4
+        ChatFlags_Console = 8
     };
 
     int    pID;
@@ -229,10 +290,9 @@ public:
 
     bool renameAllowed_;     //!< specifies if the player is allowed to rename or not, does not know about votes.
 
-    nSpamProtection chatSpam_;
-    
-    eChatLastSaid lastSaid_; //!< last said information
-    eShuffleSpamTester shuffleSpam;
+    nSpamProtection & GetChatSpam();       //!< chat volume spam
+    eChatLastSaid & GetLastSaid();         //!< last said information
+    eShuffleSpamTester & GetShuffleSpam(); //!< shuffle message spam
 
     ePlayerNetID(int p=-1);
     ePlayerNetID(nMessage &m);
@@ -241,6 +301,11 @@ public:
     virtual bool ActionOnQuit();
     virtual void ActionOnDelete();
 
+    // Check if a player can be respawned. Relaying on team alone is not enough.
+    // If a player enters as spectator, they are still assumed to be on a team.
+    // When a player is suspeded they are also on a team until the end of the round.
+    bool CanRespawn() const { return currentTeam && suspended_ == 0 && ! spectating_; }
+ 
     // chatting
     bool IsChatting() const { return chatting_; }
     void SetChatting ( ChatFlags flag, bool chatting );
@@ -256,6 +321,7 @@ public:
     eTeam* NextTeam()    const { return nextTeam; }				// return the team I will be next round
     eTeam* CurrentTeam() const { return currentTeam; }		// return the team I am in
     int  TeamListID() const { return teamListID; }		// return my position in the team
+    void SetShuffleWish( int pos ); 	                 //!< sets a desired team position
     void FindDefaultTeam();									// look for a good default team for us
     void SetTeamForce(eTeam* team );                 	// register me in the given team without checks
     void SetTeam(eTeam* team);          	// register me in the given team (callable on the server)
@@ -275,6 +341,7 @@ public:
     virtual void 			PrintName(tString &s) const;
 
     virtual bool 			AcceptClientSync() const;
+    static  bool 			AcceptClientSyncStatic();
     virtual void 			WriteSync(nMessage &m);
     virtual void 			ReadSync(nMessage &m);
     virtual nDescriptor&	CreatorDescriptor() const;
@@ -306,13 +373,13 @@ public:
     bool IsActive() const { return !disconnected; }
 
     bool IsSilenced( void ) const { return silenced_; }
-    void SetSilenced( bool silenced ) { silenced_ = silenced; }
+    void SetSilenced( bool silenced ); // { silenced_ = silenced; }
+
+    // only for the menu
     bool& AccessSilenced( void ) { return silenced_; }
 
     bool IsSuspended ( void ) { return suspended_ > 0; }
 
-    eVoter * GetVoter() const {return voter_;}     // returns our voter
-    void CreateVoter();						// create our voter or find it
     static void SilenceMenu();				// menu where you can silence players
     static void PoliceMenu();				// menu where you can silence and kick players
 
@@ -333,6 +400,8 @@ public:
     static void UpdateShuffleSpamTesters();    //<! Reset shuffle spam checks
     void LogScoreDifference();           //<! Logs accumulated scores since the last call to ResetScoreDifferences() to ladderlog.txt
 
+    void AnalyzeTiming( REAL timing );   //<! analzye a timing event for timebot detection
+
     static void SortByScore(); // brings the players into the right order
     static tString Ranking( int MAX=12, bool cut = true );     // returns a ranking list
     static void RankingLadderLog();     // writes a small ranking list to ladderlog
@@ -343,6 +412,8 @@ public:
     void GreetHighscores(tString &s); // tell him his positions in the
     // highscore lists (defined in game.cpp)
 
+    static ePlayerNetID * ReadPlayer( std::istream & s ); //!< reads a player from the stream
+
     static void Update();           // creates ePlayerNetIDs for new players
     // and destroys those of players that have left
 
@@ -399,6 +470,7 @@ private:
 
     REAL            wait_;                  //!< time in seconds WaitToLeaveChat() will wait for this player
 
+    void CreateVoter();						// create our voter or find it
     void			MyInitAfterCreation();
 
 protected:
@@ -439,7 +511,7 @@ private:
     inline ePlayerNetID & SetColoredName( tColoredString const & coloredName ); //!< Sets this player's name, cleared by the server. Use this for onscreen screen display.
 
     //! accesses the suspension count
-    int & AccessSuspended();
+    // int & AccessSuspended();
 
     //! returns the suspension count
     int GetSuspended() const;
