@@ -32,6 +32,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "tSysTime.h"
 #include "tLocale.h"
 #include "tCommandLine.h"
+#include "nSocket.h"
 #include  <time.h>
 
 REAL save_interval = 300.0f;
@@ -46,8 +47,6 @@ static tSettingItem< int > mp( "MASTER_PORT", master_port );
 REAL master_idle = 2;
 static tSettingItem< REAL > mi( "MASTER_IDLE", master_idle );
 
-extern void sn_ReceiveFromControlSocket();
-
 // console with filter for better machine readable log format
 class nConsoleDateFilter:public tConsoleFilter{
 private:
@@ -105,11 +104,13 @@ int main(int argc, char** argv)
     {
         nServerInfo::RunMaster();
 
-        tAdvanceFrame( 10000 );
+        sn_BasicNetworkSystem.Select( .1f );
+        tAdvanceFrame();
         nTimeAbsolute time = tSysTimeFloat();
 
         sn_Receive();
         sn_ReceiveFromControlSocket();
+        sn_SendPlanned();
 
         static bool queryGoesOn = true;
         if (queryGoesOn && time > querytimeout)
