@@ -20,7 +20,7 @@ GNU General Public License for more details.
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-  
+
 ***************************************************************************
 
 */
@@ -28,6 +28,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "rFont.h"
 #include "rScreen.h"
 #include "tConfiguration.h"
+#include "tColor.h"
 #include <ctype.h>
 
 #ifndef DEDICATED
@@ -55,23 +56,37 @@ rFont rFont::s_defaultFontSmall(lala_defaultFontSmall, 32,5/128.0,9/128.0,1/128.
 rFont rFont::s_defaultFontSmall("Anonymous/original/textures/font_s.png", 32,5/128.0,9/128.0,1/128.0);
 */
 
+#ifndef DEDICATED
+//! like strnlen, but that's nonstandard :-(
+static size_t my_strnlen(char const *c, size_t i) {
+	char const *begin = c;
+	char const *end = c + i;
+	for(; *c && c != end; ++c) ;
+	return c - begin;
+}
+#endif
+
 static rFont sr_lowerPartFont("textures/font_extra.png");
 rFont rFont::s_defaultFont("textures/font.png", &sr_lowerPartFont);
-rFont rFont::s_defaultFontSmall("textures/font_s.png",32,5/128.0,9/128.0,1/128.0);
+rFont rFont::s_defaultFontSmall("textures/font_s.png",32,5/128.0,9/128.0,1/128.0, -1);
 //rFont rFont::s_defaultFontSmall("textures/Font.png",0,16/256.0,32/256.0);
 //rFont rFont::s_defaultFontSmall("textures/Font.png",0,1/16.0,1/8.0);
 
-rFont::rFont(const char *fileName,int Offset,REAL CWidth,REAL CHeight,REAL op, rFont *lower):
+rFont::rFont(const char *fileName,int Offset,REAL CWidth,REAL CHeight,REAL op, int border, rFont *lower):
         rFileTexture(rTextureGroups::TEX_FONT,fileName,0,0),
         offset(Offset),cwidth(CWidth),cheight(CHeight),
-        onepixel(op),lowerPart(lower)
-{StoreAlpha();}
+        onepixel(op),borderExtension(border), lowerPart(lower)
+{
+    StoreAlpha();
+}
 
 rFont::rFont(const char *fileName, rFont *lower):
         rFileTexture(rTextureGroups::TEX_FONT,fileName,0,0),
         offset(0),cwidth(1/16.0),cheight(1/8.0),
         onepixel(1/256.0),lowerPart(lower)
-{StoreAlpha();}
+{
+    StoreAlpha();
+}
 
 rFont::~rFont(){}
 
@@ -94,18 +109,18 @@ void rFont::ProcessImage( SDL_Surface * surface )
     // pre-blend alpha values
     GLubyte *pixels =reinterpret_cast<GLubyte *>(surface->pixels);
 
-    if(surface->format->BytesPerPixel == 4)
+    if (surface->format->BytesPerPixel == 4)
     {
-        for(int i=surface->w*surface->h-1;i>=0;i--){
+        for (int i=surface->w*surface->h-1;i>=0;i--){
             GLubyte alpha=pixels[4*i+3];
             pixels[4*i  ] = (alpha * pixels[4*i  ]) >> 8;
             pixels[4*i+1] = (alpha * pixels[4*i+1]) >> 8;
             pixels[4*i+2] = (alpha * pixels[4*i+2]) >> 8;
         }
     }
-    else if(surface->format->BytesPerPixel == 2)
+    else if (surface->format->BytesPerPixel == 2)
     {
-        for(int i=surface->w*surface->h-1;i>=0;i--){
+        for (int i=surface->w*surface->h-1;i>=0;i--){
             GLubyte alpha=pixels[2*i+1];
             pixels[2*i  ] = (alpha * pixels[2*i  ]) >> 8;
         }
@@ -121,10 +136,16 @@ void rFont::OnSelect( bool enforce )
         // abort. It makes no sense to continue without a font.
         tERR_ERROR( "Font file " << this->GetFileName() << " could not be loaded.");
     }
+
+#ifndef DEDICATED
+    // wrap around so we can use the transparent pixel on the right side on the left
+    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
+#endif
 }
 
 // displays c
 #ifndef DEDICATED
+static rFont * sr_lastSelected = 0;
 void rFont::Render(unsigned char c,REAL left,REAL top,REAL right,REAL bot){
     //  if (c > 128 && this == &rFont::s_defaultFont)
     //rFont::s_defaultFontSmall.Render(c, left, top, right, bot);
@@ -143,7 +164,7 @@ void rFont::Render(unsigned char c,REAL left,REAL top,REAL right,REAL bot){
 
         REAL ttop=y*cheight+pix;
         REAL tbot=(y+1)*cheight-pix;
-        REAL tleft=x*cwidth+pix;
+        REAL tleft=x*cwidth+borderExtension*pix;
         REAL tright=(x+1)*cwidth-pix;
 
         rFont* select = this;
@@ -153,45 +174,66 @@ void rFont::Render(unsigned char c,REAL left,REAL top,REAL right,REAL bot){
             ttop -= 1;
             select = select->lowerPart;
         }
-        select->Select(true);
-
+        if ( sr_lastSelected != select )
+        {
+            RenderEnd(true);
+            select->Select(true);
+            sr_lastSelected = select;
+        }
+        
         BeginQuads();
-        glTexCoord2f(tleft,ttop);
-        glVertex2f(   left, top);
-
-        glTexCoord2f(tright,ttop);
-        glVertex2f(   right ,top);
 
         glTexCoord2f(tright,tbot);
         glVertex2f(   right, bot);
 
+        glTexCoord2f(tright,ttop);
+        glVertex2f(   right ,top);
+
+        glTexCoord2f(tleft,ttop);
+        glVertex2f(   left, top);
+
         glTexCoord2f(tleft,tbot);
         glVertex2f(   left, bot);
-        RenderEnd();
     }
 }
 #endif
 
 // **************************************************
 
-static REAL sr_bigFontThresholdWidth  = 12;
-static REAL sr_bigFontThresholdHeight = 24;
+static REAL sr_bigFontThresholdWidth  = 10;
+static REAL sr_bigFontThresholdHeight = 20;
 
 static tSettingItem< REAL > sr_bigFontThresholdWidthConf(  "FONT_BIG_THRESHOLD_WIDTH", sr_bigFontThresholdWidth );
 static tSettingItem< REAL > sr_bigFontThresholdHeightConf( "FONT_BIG_THRESHOLD_HEIGHT", sr_bigFontThresholdHeight );
 
+static REAL sr_smallFontThresholdWidth  = 5;
+static REAL sr_smallFontThresholdHeight = 8;
+
+static tSettingItem< REAL > sr_smallFontThresholdWidthConf(  "FONT_SMALL_THRESHOLD_WIDTH", sr_smallFontThresholdWidth );
+static tSettingItem< REAL > sr_smallFontThresholdHeightConf( "FONT_SMALL_THRESHOLD_HEIGHT", sr_smallFontThresholdHeight );
+
 rTextField::rTextField(REAL Left,REAL Top,
                        REAL Cwidth,REAL Cheight,
                        rFont *f)
         :parIndent(0),
         left(Left),top(Top),cwidth(Cwidth),cheight(Cheight),
-F(f),x(0),y(0),realx(0),cursor(0),cursorPos(0){
+        F(f),x(0),y(0),realx(0),cursor(0),cursorPos(0){
     if ( cwidth*sr_screenWidth < sr_bigFontThresholdWidth*2 || cheight*sr_screenHeight < sr_bigFontThresholdHeight*2 )
         F=&rFont::s_defaultFontSmall;
-    if (cwidth*sr_screenWidth<10)
-        cwidth=10/REAL(sr_screenWidth);
-    if (cheight*sr_screenHeight<18)
-        cheight=18/REAL(sr_screenHeight);
+    if (cwidth*sr_screenWidth <= sr_smallFontThresholdWidth*2 + 1E-4)
+    {
+        cwidth=sr_smallFontThresholdWidth*2/REAL(sr_screenWidth);
+
+        // try to place font at exact pixels
+        left = Pixelize(left, sr_screenWidth);
+    }
+    if (cheight*sr_screenHeight <= sr_smallFontThresholdHeight*2 + 1E-4)
+    {
+        cheight=sr_smallFontThresholdHeight*2/REAL(sr_screenHeight);
+
+        // try to place font at exact pixels
+        top = Pixelize(top, sr_screenHeight);
+    }
 
     color_ = defaultColor_;
 
@@ -199,7 +241,7 @@ F(f),x(0),y(0),realx(0),cursor(0),cursorPos(0){
 
     buffer.SetLen(0);
     /*
-    top=(int(top*sr_screenHeight)+.5)/REAL(sr_screenHeight);  
+    top=(int(top*sr_screenHeight)+.5)/REAL(sr_screenHeight);
     left=(int(left*sr_screenWidth)+.5)/REAL(sr_screenWidth);
     */
 
@@ -207,11 +249,28 @@ F(f),x(0),y(0),realx(0),cursor(0),cursorPos(0){
     cursor_y = -100;
 }
 
+REAL rTextField::AspectWidthMultiplier()
+{
+    return std::min(((4.0f/3.0f)*sr_screenHeight)/sr_screenWidth,1.0f);
+}
+
+REAL rTextField::AspectHeightMultiplier()
+{
+    return std::min(((3.0f/4.0f)*sr_screenWidth)/sr_screenHeight,1.0f);
+}
+
+REAL rTextField::Pixelize(REAL xy, int WidthHeight)
+{
+    auto pixelIn = static_cast<int>(.5f * xy * WidthHeight);
+    return (2.0f*(pixelIn+.5f))/WidthHeight;
+}
+
 
 rTextField::~rTextField(){
-    FlushLine();
-
 #ifndef DEDICATED
+    FlushLine();
+    RenderEnd();
+
     if (cursor && sr_glOut){
         if (cursor==2)
             glColor4f(1,1,1,.5);
@@ -229,13 +288,6 @@ rTextField::~rTextField(){
 #endif
 }
 
-// minimal tolerated values of font color before a white background is rendered
-static REAL sr_minR = .5, sr_minG = .5, sr_minB =.5, sr_minTotal = .7;
-tSettingItem< REAL > sr_minRConf( "FONT_MIN_R", sr_minR );
-tSettingItem< REAL > sr_minGConf( "FONT_MIN_G", sr_minG );
-tSettingItem< REAL > sr_minBConf( "FONT_MIN_B", sr_minB );
-tSettingItem< REAL > sr_minTotalConf( "FONT_MIN_TOTAL", sr_minTotal );
-
 void rTextField::FlushLine(int len,bool newline){
 #ifndef DEDICATED
     // reload textures if alpha blending changed
@@ -260,13 +312,12 @@ void rTextField::FlushLine(int len,bool newline){
     if (sr_glOut)
     {
         // render bright background
-        if ( r < sr_minR && g < sr_minG && b < sr_minG || r+g+b < sr_minTotal )
+        if ( color_.IsDark() )
         {
+            RenderEnd(true);
+            glDisable(GL_TEXTURE_2D);
             if ( sr_alphaBlend )
             {
-
-                glDisable(GL_TEXTURE_2D);
-
                 glColor4f( blendColor_.r_, blendColor_.g_, blendColor_.b_, a * blendColor_.a_ );
 
                 REAL l=left+realx*cwidth;
@@ -275,14 +326,14 @@ void rTextField::FlushLine(int len,bool newline){
                 REAL b=t - cheight;
 
                 BeginQuads();
-                glVertex2f(   l, t);
 
-                glVertex2f(   r ,t);
+                glVertex2f(   l, b);
 
                 glVertex2f(   r, b);
 
-                glVertex2f(   l, b);
-                RenderEnd();
+                glVertex2f(   r ,t);
+
+                glVertex2f(   l, t);
             }
             else
             {
@@ -290,22 +341,29 @@ void rTextField::FlushLine(int len,bool newline){
                 if ( g < .5 ) g = .5;
                 if ( b < .5 ) b = .5;
             }
+            RenderEnd(true);
+            glEnable(GL_TEXTURE_2D);
         }
 
-        glColor4f(r * blendColor_.r_,g * blendColor_.g_,b * blendColor_.b_,a * blendColor_.a_);
-    }
 
-    for(i=0;i<=len;i++){
-        REAL l=left+realx*cwidth;
-        REAL t=top-y*cheight;
-
-        if (0 <= cursorPos--){
-            cursor_x=l;
-            cursor_y=t;
+        if ( len > 0 )
+        {
+            RenderEnd(true);
+            glColor4f(r * blendColor_.r_,g * blendColor_.g_,b * blendColor_.b_,a * blendColor_.a_);
+            sr_lastSelected = 0;
         }
-        if (i<len){
-            F->Render(buffer[realx],l,t,l+cwidth,t-cheight);
-            realx++;
+        for (i=0;i<=len;i++){
+            REAL l=left+realx*cwidth;
+            REAL t=top-y*cheight;
+            
+            if (0 <= cursorPos--){
+                cursor_x=l;
+                cursor_y=t;
+            }
+            if (i<len){
+                F->Render(buffer[realx],l,t,l+cwidth,t-cheight);
+                realx++;
+            }
         }
     }
 
@@ -335,7 +393,7 @@ void rTextField::FlushLine(bool newline){
 
 inline void rTextField::WriteChar(unsigned char c)
 {
-    switch(c){
+    switch (c){
     case('\n'):
                     FlushLine();
         buffer.SetLen(0);
@@ -383,41 +441,21 @@ rTextField & rTextField::operator<<(unsigned char c){
 }
 */
 
-static REAL CTR(int x){
-    return x/255.0;
-}
-
-static char hex_array[]="0123456789abcdef";
-
-char int_to_hex(int i){
-    if (i<0 || i >15)
-        return 'Q';
-    else
-        return hex_array[i];
-}
-
-int hex_to_int(char c){
-    int ret=0;
-    for(int i=15;i>=0;i--)
-        if (hex_array[i]==c)
-            ret=i;
-    return ret;
-}
-
-rTextField & rTextField::StringOutput(const char * c, ColorMode colorMode)
+rTextField & rTextField::StringOutput(const char * c, ColorMode colorMode )
 {
+#ifndef DEDICATED
     // run through string
     while (*c!='\0')
     {
         // break line if next space character is too far away
-        if ( isspace(*c) )
+        if ( isblank(*c) )
         {
             // count number of nonblank characters following
             char const * nextSpace = c+1;
             int wordLen = 0;
-            while ( *nextSpace != '\0' && !isspace(*nextSpace) )
+            while ( *nextSpace != '\0' && *nextSpace != '\n' && !isblank(*nextSpace) )
             {
-                if (*nextSpace=='0' && strlen(nextSpace)>=8 && nextSpace[1]=='x' && colorMode != COLOR_IGNORE )
+                if (*nextSpace=='0' && my_strnlen(nextSpace, 8)>=8 && nextSpace[1]=='x' && colorMode != COLOR_IGNORE )
                 {
                     // skip color code
                     nextSpace += 8;
@@ -436,6 +474,11 @@ rTextField & rTextField::StringOutput(const char * c, ColorMode colorMode)
                 // no. Skip to the next line
                 WriteChar('\n');
                 c++;
+                for ( int i = parIndent-1; i >= 0; --i )
+                {
+                    WriteChar(' ');
+                    cursorPos++;
+                }
                 continue;
             }
         }
@@ -444,26 +487,27 @@ rTextField & rTextField::StringOutput(const char * c, ColorMode colorMode)
         if ( x >= width )
         {
             WriteChar('\n');
+            cursorPos += 1;
         }
 
         // detect presence of color code
-        if (*c=='0' && strlen(c)>=8 && c[1]=='x' && colorMode != COLOR_IGNORE )
+        if (*c=='0' && my_strnlen(c, 8)>=8 && c[1]=='x' && colorMode != COLOR_IGNORE )
         {
+            tColor color;
+            bool use = false;
+
             if ( 0 ==strncmp(c,"0xRESETT",8) )
             {
                 // color reset to default requested
-                ResetColor();
+                color = defaultColor_;
+                use = true;
             }
             else
             {
                 // found! extract colors
-                FlushLine(false);
-
-                cursorPos-=7;
-
-                color_.r_=CTR(hex_to_int(c[2])*16+hex_to_int(c[3]));
-                color_.g_=CTR(hex_to_int(c[4])*16+hex_to_int(c[5]));
-                color_.b_=CTR(hex_to_int(c[6])*16+hex_to_int(c[7]));
+                cursorPos-=8;
+		color = tColor( c );
+                use = true;
             }
 
             // advance
@@ -473,43 +517,84 @@ rTextField & rTextField::StringOutput(const char * c, ColorMode colorMode)
             }
             else
             {
-                c++;
+                // write color code out
+                cursorPos+=8;
+                for(int i=7; i>=0;--i)
+                    WriteChar(*(c++));
+            }
+
+            // apply color
+            if ( use )
+            {
+                FlushLine(false);
+                cursorPos++;
+                color_ = color;
             }
         }
         else
             // normal operation: add char
             WriteChar(*(c++));
     }
+
+    RenderEnd( true );
+#endif
+
     return *this;
 }
 
-void DisplayText(REAL x,REAL y,REAL w,REAL h,const char *text,int center,int cursor,int cursorPos){
+void DisplayText(REAL x,REAL y,REAL w,REAL h,const char *text,int center,int cursor,int cursorPos, rTextField::ColorMode colorMode ){
     int colorlen = strlen(text);
-    for (unsigned int i=0; i<strlen(text); i++)
+
+    if ( colorMode == rTextField::COLOR_USE )
     {
-        if (text[i] == '0' && text[i+1] == 'x')
-            colorlen=colorlen-8;
+        for(char const *c = text; *c != 0; ++c)
+        {
+            if(*c == '0' && c[1] == 'x')
+                colorlen -= 8;
+        }
     }
 
+    // calculate top position so that does not move when we shrink the font
+    REAL top = y + h*.5;
+
     // shrink fields that don't fit the screen
-    REAL maxw = 1.95;
-    if ( colorlen * w > maxw )
+    REAL availw = 1.9f;
+    if (center < 0) availw = (.9f-x);
+    if (center > 0) availw = (x + .9f);
+    if ( colorlen * w > availw )
     {
-        h *= maxw/(colorlen * w);
-        w = maxw/REAL(colorlen);
+        h *= availw/(colorlen * w);
+        w = availw/REAL(colorlen);
     }
 
     rTextField c(x-(center+1)*colorlen*.5*w,y+h*.5,w,h);
     if (center==-1)
-        c.SetWidth(int((1-x)/w));
+        c.SetWidth(int((.95-x)/c.GetCWidth()));
     else
         c.SetWidth(10000);
 
+    // did the text field enlarge the font? If yes, there will be wrapping; better make some more room
+    if ( c.GetWidth() < colorlen )
+    {
+        c.SetTop( top );
+    }
+
     c.SetIndent(5);
     if (cursor)
         c.SetCursor(cursor,cursorPos);
-    c << text;
+    c.StringOutput(text, colorMode );
 }
+
+void DisplayTextAutoWidth(REAL x, REAL y, const char *text, REAL h, int center, int cursor, int cursorPos, rTextField::ColorMode colorMode)
+{
+    DisplayText(x, y, h*(rCWIDTH_NORMAL/rCHEIGHT_NORMAL)*rTextField::AspectWidthMultiplier(), h, text, center, cursor, cursorPos, colorMode);
+}
+
+void DisplayTextAutoHeight(REAL x, REAL y, const char *text, REAL w, int center, int cursor, int cursorPos, rTextField::ColorMode colorMode)
+{
+    DisplayText(x, y, w, w*(rCHEIGHT_NORMAL/rCWIDTH_NORMAL)*rTextField::AspectHeightMultiplier(), text, center, cursor, cursorPos, colorMode);
+}
+
 // *******************************************************************************************
 // *
 // *	GetDefaultColor
@@ -553,6 +638,13 @@ void rTextField::GetDefaultColor( tColor & defaultColor )
 void rTextField::SetDefaultColor( tColor const & defaultColor )
 {
     defaultColor_ = defaultColor;
+    if ( !sr_alphaBlend )
+    {
+        defaultColor_.r_ *= defaultColor_.a_;
+        defaultColor_.g_ *= defaultColor_.a_;
+        defaultColor_.b_ *= defaultColor_.a_;
+        defaultColor_.a_ = 1;
+    }
     blendColor_ = tColor();
 }
 
@@ -599,10 +691,15 @@ void rTextField::GetBlendColor( tColor & blendColor )
 void rTextField::SetBlendColor( tColor const & blendColor )
 {
     blendColor_ = blendColor;
+    if ( !sr_alphaBlend )
+    {
+        blendColor_.r_ *= blendColor_.a_;
+        blendColor_.g_ *= blendColor_.a_;
+        blendColor_.b_ *= blendColor_.a_;
+        blendColor_.a_ = 1;
+    }
 }
 
 tColor rTextField::defaultColor_;
 tColor rTextField::blendColor_;
 
-
-
