@@ -20,7 +20,7 @@ GNU General Public License for more details.
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-  
+
 ***************************************************************************
 
 */
@@ -30,6 +30,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "config.h"
 
 #include "rTexture.h"
+#include "rDisplayList.h"
 #include "tString.h"
 #include "rScreen.h"
 #include "tDirectories.h"
@@ -444,6 +445,8 @@ rISurfaceTexture::~rISurfaceTexture( void )
 #ifndef DEDICATED
     if (tint_ > 0)
     {
+        rDisplayList::ClearAll();
+
         glDeleteTextures(1,&tint_);
         tint_ = 0;
     }
@@ -477,6 +480,12 @@ void rISurfaceTexture::ProcessImage( SDL_Surface * surface )
 void rISurfaceTexture::Upload( rSurface & surface )
 {
 #ifndef DEDICATED
+#ifndef GL_CLAMP_TO_EDGE
+// GL_CLAMP_TO_EDGE was observed undefined in Windows; should be the same as GL_CLAMP.
+#define GL_CLAMP_TO_EDGE GL_CLAMP
+#endif
+
+    sr_LockSDL();
     GLenum texformat = surface.GetFormat();
     SDL_Surface * tex = surface.GetSurface();
     tASSERT( tex );
@@ -508,7 +517,9 @@ void rISurfaceTexture::Upload( rSurface & surface )
 
     gluBuild2DMipmaps(GL_TEXTURE_2D,format,tex->w,tex->h,
                       texformat,GL_UNSIGNED_BYTE,tex->pixels);
-#endif
+
+    sr_UnlockSDL();
+ #endif
 }
 
 // ******************************************************************************************
@@ -526,6 +537,8 @@ void rISurfaceTexture::OnSelect( bool enforce )
 #ifndef DEDICATED
     if(sr_glOut)
     {
+        RenderEnd(true);
+
         int texmod=rTextureGroups::TextureMode[group_];
         if (enforce && texmod<0) texmod=GL_NEAREST_MIPMAP_NEAREST;
 
@@ -538,7 +551,8 @@ void rISurfaceTexture::OnSelect( bool enforce )
             // std::cerr << "loading texture " << fileName << ':' << tint << "\n";
 
             if (texmod>0){
-                RenderEnd(true);
+                // don't generate textures inside display lists
+                rDisplayList::Cancel();
 
                 glGenTextures(1, &tint_);
                 glBindTexture(GL_TEXTURE_2D,tint_);
@@ -621,6 +635,8 @@ void rISurfaceTexture::OnUnload( void )
 #ifndef DEDICATED
     if (tint_ > 0)
     {
+        rDisplayList::ClearAll();
+
         // std::cerr << "unloading texture " << fileName << ':' << tint_ << "\n";
         glDeleteTextures(1,&tint_);
         tint_ = 0;
@@ -763,9 +779,13 @@ void rSurfaceTexture::OnSelect()
 bool rISurfaceTexture::s_reportErrors_=false;
 tList<rITexture> rITexture::s_textures_;
 
-int rTextureGroups::TextureMode[rTextureGroups::TEX_GROUPS];
+int rTextureGroups::TextureMode[rTextureGroups::TEX_GROUPS]
+#ifndef DEDICATED
+={GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR }
+#endif
+;
 
-char *rTextureGroups::TextureGroupDescription[rTextureGroups::TEX_GROUPS]=
+char const * rTextureGroups::TextureGroupDescription[rTextureGroups::TEX_GROUPS]=
     {
         "$texture_mode_0_help",
         "$texture_mode_1_help",
