@@ -935,12 +935,15 @@ tString extraConfig("NONE");
 class tExtraConfigCommandLineAnalyzer: public tCommandLineAnalyzer
 {
 private:
-    virtual bool DoAnalyze( tCommandLineParser & parser )
+    bool DoAnalyze( tCommandLineParser & parser, int pass ) override
     {
+        if(pass > 0)
+            return false;
+
         return parser.GetOption(extraConfig, "--extraconfig", "-e");
     }
 
-    virtual void DoHelp( std::ostream & s )
+    void DoHelp( std::ostream & s ) override
     {                                      //
         s << "-e, --extraconfig            : open an extra configuration file after\n"
         << "                               settings_dedicated.cfg\n";
@@ -1163,3 +1166,119 @@ static tConfItemFunc st_Dummy10("SIMULATE_RECEIVE_PACKET_LOSS", &st_Dummy);
 static tConfItemFunc st_Dummy11("SIMULATE_SEND_PACKET_LOSS", &st_Dummy);
 #endif
 
+namespace
+{
+std::vector<tConfigMigration::Callback> &st_MigrationCallbacks()
+{
+    static std::vector<tConfigMigration::Callback> callbacks;
+    return callbacks;
+}
+
+#ifdef DEBUG
+struct tConfigMigrationTester
+{
+    tConfigMigrationTester()
+    {
+        tASSERT(!tConfigMigration::SavedBefore("0.2.9","0.2.9"));
+
+        auto AssertOrdered = [](char const *a, char const *b)
+        {
+            tASSERT(tConfigMigration::SavedBefore(a,b));
+            tASSERT(!tConfigMigration::SavedBefore(b,a));
+        };
+
+        AssertOrdered("0.2.8","0.2.9");
+        AssertOrdered("0.2.8","0.10.9");
+        AssertOrdered("0.4.0_alpha100","0.4.0_beta1");
+        AssertOrdered("0.4.0_beta229","0.4.0_rc1");
+        AssertOrdered("0.4.0_rc19","0.4.0");
+        AssertOrdered("0.","0.0");
+        AssertOrdered("0.","0.1");
+        AssertOrdered("0.","0.9");
+        AssertOrdered("0","0.");
+        AssertOrdered("0_","0");
+    }
+};
+
+static tConfigMigrationTester st_ConfigMigrationTester;
+#endif
+}
+
+tConfigMigration::tConfigMigration(tConfigMigration::Callback &&cb)
+{
+    st_MigrationCallbacks().push_back(std::move(cb));
+}
+
+void tConfigMigration::Migrate(const tString &savedInVersion)
+{
+    for(auto &callback: st_MigrationCallbacks())
+    {
+        callback(savedInVersion);
+    }
+}
+
+bool tConfigMigration::SavedBefore(char const *savedInVersion, char const *beforeVersion)
+{
+    char const *pA = savedInVersion, *pB = beforeVersion;
+    while(*pA && *pB)
+    {
+        // equality -> advance
+        if(*pA == *pB)
+        {
+            ++pA;
+            ++pB;
+            continue;
+        }
+
+        // non-number difference -> regular alphabetic order
+        if(!isdigit(*pA) || !isdigit(*pB))
+        {
+            return *pA < *pB;
+        }
+
+        // this leaves number differences.
+        auto lengthOfNumber = [](char const *pBegin)
+        {
+            auto *pC = pBegin;
+            while(isdigit(*pC))
+                ++pC;
+            return pC - pBegin;
+        };
+
+        // longer numbers are bigger
+        auto const lA = lengthOfNumber(pA);
+        auto const lB = lengthOfNumber(pB);
+        if(lA < lB)
+            return true;
+        if(lA > lB)
+            return false;
+
+        // equal length numbers can be compared digit by digit
+        while(isdigit(*pA))
+        {
+            if(*pA == *pB)
+            {
+                ++pA;
+                ++pB;
+                continue;
+            }
+
+            return *pA < *pB;
+        }
+    }
+    if(!*pA && !*pB)
+        return false;
+
+    // reversed order if one version string is longer than the other;
+    // we want _ to be less than nothing, but . more, and they sit on
+    // opposite sides of the digits.
+    if(!*pA)
+    {
+        return '9' >= *pB;
+    }
+    else
+    {
+        tASSERT(!*pB);
+        return '9' < *pA;
+    }
+}
