@@ -41,20 +41,25 @@ public:
 
     bool Open    ( std::ofstream& f,
                    const char* filename,
-                   std::ios::openmode mode = std::ios::out ) const; // opens a file to write
+                   std::ios::openmode mode = std::ios::out,
+                   bool sensitive = false ) const; // opens a file to write
 
     tString GetReadPath   ( const char* filename   ) const; // finds the full path to a file to read
     tString GetWritePath  ( const char* filename   ) const; // finds the full path to a file to write
     static bool IsValidPath( char const * filename ); //!< checks whether filename is valid, i.e. does not endanger system security.
 
-    tPath(){};
-    virtual ~tPath(){};
+    tString GetPaths(void) const; //!< Puts all paths into a tString for outputting to the user
+    tString GetPaths(char const * delimiter, char const * finalizer) const; //!< Puts all paths into a tString for outputting to the user
+
+    tPath(){}
+    virtual ~tPath(){}
 protected:
-    virtual void    Paths ( tArray< tString >& paths ) const = 0;  // maximum priority
+    virtual void    Paths ( tArray< tString >& paths ) const = 0;  // maximum priority is given to paths[0]
 };
 
 class tPathResource: public tPath {
 public:
+    tPathResource() {}
     tString GetWritePath(const char *filename) const;
     tString GetIncluded() const; //!< returns the path to the included resources
 private:
