@@ -28,6 +28,8 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #ifndef ArmageTron_tLinkedList_H
 #define ArmageTron_tLinkedList_H
 
+#include "tError.h"
+
 #include <stdlib.h>
 
 class tListItemBase{
@@ -41,11 +43,16 @@ public:
     void Remove();
     void Insert(tListItemBase *&a);
 
+    tListItemBase():next(NULL),anchor(NULL)                 {}
     tListItemBase(tListItemBase *&a):next(NULL),anchor(NULL){Insert(a);}
     virtual ~tListItemBase()                                {Remove();}
 
+    //! returns the next list element
     tListItemBase *Next()                             {return next;}
 
+    //! returns true if this object is in a list
+    bool IsInList() const {return anchor;}
+
     int Len();
     void Sort( Comparator* comparator );
 };
@@ -62,7 +69,12 @@ int st_Compare( const tListItemBase* a, const tListItemBase* b )
 
 template <class T> class tListItem:public tListItemBase{
 public:
-    tListItem(T *&a):tListItemBase(reinterpret_cast<tListItemBase*&>(a)){};
+    tListItem():tListItemBase()
+    { 
+        // this class only works under this condition:
+        tASSERT( static_cast< tListItemBase * >( ( T * )(NULL)  ) == NULL );
+    }
+    tListItem(T *&a):tListItemBase(reinterpret_cast<tListItemBase*&>(a)){}
     T *Next(){return reinterpret_cast<T*>(next);}
 
     template< typename comparator >
@@ -70,6 +82,21 @@ public:
     {
         tListItemBase::Sort( &st_Compare< T, comparator> );
     }
+
+    void Insert(tListItem *&a)
+    {
+        tListItemBase::Insert( reinterpret_cast<tListItemBase*&>(a) );
+    }
+
+    void Insert(T *&a)
+    {
+        tListItemBase::Insert( reinterpret_cast<tListItemBase*&>(a) );
+    }
+
+    void Insert(tListItemBase *&a)
+    {
+        tListItemBase::Insert( a );
+    }
 };
 
 #endif
