@@ -35,6 +35,8 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include <string>
 #include <map>
 
+const tString st_internalEncoding("latin1");
+
 class tLocaleSubItem; // identifies a single string in a single language
 
 static tArray<tString> st_TemplateParameters;
@@ -594,7 +596,18 @@ void tOutput::AddLocale(const char *x)
 {
     tNEW(tOutputItemLocale)(*this, tLocale::Find(x));
 }
-
+void tOutput::AddString(char const * locale)
+{
+    int len = strlen(locale);
+    if (len == 0)
+        return;
+    if (len == 1 && locale[0] == ' ')
+        tNEW(tOutputItemSpace)(*this);
+    else if (locale[0] == '$')
+        tNEW(tOutputItemLocale)(*this, tLocale::Find(locale+1));
+    else
+        tNEW(tOutputItem<tString>)(*this, tString(locale));
+}
 
 tOutput & tOutput::SetTemplateParameter(int num, const char *parameter)
 {
@@ -731,16 +744,7 @@ tOutput& operator << (tOutput &o, char *locale)
 
 // and a special implementation for the locales and strings:
 tOutput& operator << (tOutput &o, const char *locale){
-    int len = strlen(locale);
-    if (len == 0)
-        return o;
-    if (len == 1 && locale[0] == ' ')
-        tNEW(tOutputItemSpace)(o);
-    else if (locale[0] == '$')
-        tNEW(tOutputItemLocale)(o, tLocale::Find(locale+1));
-    else
-        tNEW(tOutputItem<tString>)(o, tString(locale));
-
+    o.AddString(locale);
     return o;
 }
 
