@@ -89,10 +89,10 @@ static bool reported=false;
 #ifdef HAVE_LIBZTHREAD
 #include <zthread/FastRecursiveMutex.h>
 
-static ZThread::FastRecursiveMutex st_mutex;
+typedef ZThread::FastRecursiveMutex MUTEX;
 #elif defined(HAVE_PTHREAD)
 #include "pthread-binding.h"
-static tPThreadRecursiveMutex st_mutex;
+typedef tPThreadRecursiveMutex MUTEX;
 #else
 class tMockMutex
 {
@@ -101,20 +101,41 @@ public:
     void release(){}
 };
 
-static tMockMutex st_mutex;
+typedef tMockMutex MUTEX;
 #endif
 
+static bool inited=true;
+
+static MUTEX & st_CreateMutex()
+{
+    inited = false;
+    static MUTEX newMutex;
+    inited = true;
+
+#ifdef WIN32
+    InitializeCriticalSection(&mutex);
+#endif
+
+    return newMutex;
+}
+
+static MUTEX & st_Mutex()
+{
+    static MUTEX & mutex = st_CreateMutex();
+    return mutex;
+}
+
 class tBottleNeck
 {
 public:
     tBottleNeck()
     {
-        st_mutex.acquire();
+        st_Mutex().acquire();
     }
 
     ~tBottleNeck()
     {
-        st_mutex.release();
+        st_Mutex().release();
     }
 };
 
@@ -252,8 +273,6 @@ private:
     int semaphore;
 };
 
-static bool inited=false;
-
 #ifdef LEAKFINDER
 #include <fstream>
 #define MAXCHECKSUM 100001
@@ -676,11 +695,6 @@ tMemManager::tMemManager(int s,int bs):size(s),blocksize(s){
 
     tBottleNeck neck;
 
-#ifdef WIN32
-    if (!inited)
-        InitializeCriticalSection(&mutex);
-#endif
-
     inited = true;
 }
 
@@ -714,7 +728,7 @@ tMemManager::tMemManager(int s):size(s){//,blocks(1000),full_blocks(1000){
 
         std::ifstream l(leakname);
         while (l.good() && !l.eof()){
-            int cs,ln;
+            int cs{-1},ln{0};
             l >> cs >> ln;
 
             if (cs>=0 && cs < MAXCHECKSUM && (ln < leaks[cs] || leaks[cs] == 0))
@@ -728,11 +742,6 @@ tMemManager::tMemManager(int s):size(s){//,blocks(1000),full_blocks(1000){
     }
 #endif
 
-#ifdef WIN32
-    if (!inited)
-        InitializeCriticalSection(&mutex);
-#endif
-
     inited = true;
 }
 
@@ -951,7 +960,9 @@ void tMemManager::Check(){
 #define MAX_SIZE 109
 
 
-static tMemManager memman[MAX_SIZE+1]={
+static tMemManager & st_MemMan(int id)
+{
+    static tMemManager memman[MAX_SIZE+1]={
                                           tMemManager(0),
                                           tMemManager(4),
                                           tMemManager(8),
@@ -1062,7 +1073,12 @@ static tMemManager memman[MAX_SIZE+1]={
                                           tMemManager(428),
                                           tMemManager(432),
                                           tMemManager(436)
-                                      };
+    };
+
+    tASSERT(id >= 0 && id <= MAX_SIZE);
+    return memman[id];
+}
+
 
 
 void tMemManager::Dispose(tAllocationInfo const & info, void *p, bool keep){
@@ -1075,7 +1091,7 @@ void tMemManager::Dispose(tAllocationInfo const & info, void *p, bool keep){
 #ifndef DOUBLEFREEFINDER
     if (inited && block){
         tBottleNeck neck;
-        memman[size >> 2].complete_Dispose(block);
+        st_MemMan(size >> 2).complete_Dispose(block);
 #ifdef WIN32
         LeaveCriticalSection(&mutex);
 #endif
@@ -1124,7 +1140,7 @@ void *tMemMan::Alloc(tAllocationInfo const & info, size_t s){
     if (inited && s < (MAX_SIZE << 2))
     {
         tBottleNeck neck;
-        ret=memman[((s+3)>>2)].Alloc( info );
+        ret=st_MemMan(((s+3)>>2)).Alloc( info );
     }
     else
     {
@@ -1184,6 +1200,7 @@ void tMemMan::Dispose(tAllocationInfo const & info, void *p){
 
 
 void tMemManBase::Check(){
+#ifndef DONTUSEMEMMANAGER
     if (!inited)
         return;
 
@@ -1192,11 +1209,13 @@ void tMemManBase::Check(){
 #endif
 
     for (int i=MAX_SIZE;i>=0;i--)
-        memman[i].Check();
+        st_MemMan(i).Check();
 
 #ifdef WIN32
     LeaveCriticalSection(&mutex);
 #endif
+
+#endif // DONTUSEMEMMANAGER
 }
 
 /*
