@@ -20,7 +20,7 @@ GNU General Public License for more details.
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-  
+
 ***************************************************************************
 
 */
@@ -31,6 +31,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "tLocale.h"
 #include "tConfiguration.h"
 #include <ctype.h>
+#include <time.h>
 #include <string>
 #include <iostream>
 
@@ -300,13 +301,13 @@ public:
     {
         // basic stuff
         Test( "abc", "abc" );
-        
+
         // quoted
         Test( "'abc'", "abc" );
 
         // escaped
         Test( "\\'a\\\"b", "'a\"b" );
-        
+
         // chained backslashes
         Test( "\\\\", "\\" );
         Test( "\\\\\\\\", "\\\\" );
@@ -683,7 +684,7 @@ tString & tString::operator+=(const tString &s)
 //static int   st_TempStringLength = 1000;
 #endif
 
-// static int	 
+// static int
 
 class tTempStringCleanup
 {
@@ -1133,7 +1134,7 @@ tString tColoredString::RemoveColors( const char * c, bool darkonly )
     tString ret;
     int len = strlen(c);
     bool removed = false;
-    
+
     // walk through string
     while (*c!='\0'){
         // skip color codes
@@ -1148,7 +1149,7 @@ tString tColoredString::RemoveColors( const char * c, bool darkonly )
                     removed = true;
 
                 c   += 8;
-                len -= 8;	
+                len -= 8;
             }
             else if( len >= 8 )
             {
@@ -1169,9 +1170,9 @@ tString tColoredString::RemoveColors( const char * c, bool darkonly )
             len--;
         }
     }
-    
+
     // st_Breakpoint();
-    
+
     return removed ? RemoveColors( ret, darkonly ) : ret;
 }
 
@@ -1285,11 +1286,13 @@ public:
 
         // leave ASCII characters as they are
         // for (i=127; i>=32; --i)
-        // no, leave all ISO Latin 1 characters as they are
-        for (i=255; i>=32; --i)
-        {
-            filter[i] = i;
-        }
+        // no, leave all visible ISO Latin 1 characters as they are
+        for ( i = 32; i <= 126; ++i )
+            filter[ i ] = i;
+        for ( i = 161; i <= 255; ++i )
+            filter[ i ] = i;
+
+        filter[ 160 ] = ' '; // non-breaking space
 
         // map return and tab to space
         SetMap('\n',' ');
@@ -1404,7 +1407,7 @@ void tColoredString::RemoveHex( void )
     //    newS.RemoveHex();
     //    }
     //    }
-    //    } 
+    //    }
     //this = newS; */
 }
 
@@ -1502,7 +1505,7 @@ static tConfItemLine st_wordDelimiters( "WORD_DELIMITERS", delimiters );
 //!
 // *******************************************************************************************
 
-int tString::PosWordRight( int start ) {
+int tString::PosWordRight( int start ) const {
     int nextDelimiter = strcspn( SubStr( start, Len() ), delimiters );
     int toMove = nextDelimiter;
 
@@ -1534,7 +1537,7 @@ int tString::PosWordRight( int start ) {
 //!
 // *******************************************************************************************
 
-int tString::PosWordLeft( int start ) {
+int tString::PosWordLeft( int start ) const {
     return -1 * Reverse().PosWordRight( Len() - start - 1 );
 }
 
@@ -1548,7 +1551,7 @@ int tString::PosWordLeft( int start ) {
 //!
 // *******************************************************************************************
 
-tString tString::Reverse() {
+tString tString::Reverse() const {
     tString reversed;
     for ( int index = Len() - 2; index >= 0; index-- ) {
         reversed << ( *this ) ( index );
@@ -1686,7 +1689,7 @@ bool tIsInList( tString const & list_, tString const & item )
         }
 
         // check whether the match is a true list match
-        if ( 
+        if (
             ( pos == 0 || list[pos-1] == ',' || isblank(list[pos-1]) )
             &&
             ( pos + item.Len() >= list.Len() || list[pos+item.Len()-1] == ',' || isblank(list[pos+item.Len()-1]) )
@@ -1700,7 +1703,7 @@ bool tIsInList( tString const & list_, tString const & item )
             list = list.SubStr( pos + 1 );
         }
     }
-    
+
     return false;
 }
 
