@@ -154,6 +154,7 @@ static tConfItem<bool>    ab("ALPHA_BLEND",sr_alphaBlend);
 static tConfItem<bool>    ss("SMOOTH_SHADING",sr_smoothShading);
 static tConfItem<bool>    to("TEXT_OUT",sr_textOut);
 static tConfItem<bool>    fps("SHOW_FPS",sr_FPSOut);
+static tConfItem<bool>    pbt("SHOW_RECORDING_TIME",sr_RecordingTimeOut);
 // tConfItem<> ("",&);
 static tConfItem<int> fm("FLOOR_MIRROR",sr_floorMirror);
 static tConfItem<int> fd("FLOOR_DETAIL",sr_floorDetail);
@@ -280,7 +281,7 @@ public:
                 else
                     s << tOutput("$screen_size_desktop");
 
-                res_men.NewChoice( s.str().c_str(), help, size );
+                res_men.NewChoice( s.str().c_str(), "", size );
             }
 
 #endif
@@ -407,6 +408,16 @@ static uMenuItemToggle fs_dither
  "$detail_dither_help",
  sr_dither);
 
+#ifndef DEDICATED
+// from gWall.cpp
+extern bool sg_simpleTrail;
+static uMenuItemToggle sgm_simpleTrail
+(&screen_menu_detail,
+ "$detail_simple_trail_text",
+ "$detail_simple_trail_help",
+ sg_simpleTrail);
+#endif
+
 static uMenuItemSelection<int> mfd
 (&screen_menu_detail,
  "$detail_floor_text",
@@ -447,7 +458,6 @@ static tConfItem<bool> crexp("EXPLOSION",sg_crashExplosion);
 #ifndef DEDICATED
 //extern bool png_screenshot;		// from rSysdep.cpp
 //static tConfItem<bool> pns("PNG_SCREENSHOT",png_screenshot);
-#endif
 
 static uMenuItemToggle  t32b
 (&screen_menu_detail,"$detail_text_truecolor_text",
@@ -472,6 +482,10 @@ static ArmageTron_texmode_menuitem tmm3(&screen_menu_detail,
                                         rTextureGroups::TextureMode[3],true);
 
 
+uMenuItemToggle bpt2
+(&screen_menu_prefs,"$misc_recording_time_text",
+ "$misc_recording_time_help",sr_RecordingTimeOut);
+
 static uMenuItemToggle s2
 (&screen_menu_prefs,"$pref_highrim_text",
  "$pref_highrim_help",sr_highRim);
@@ -629,14 +643,8 @@ uMenuItemToggle hud2
 (&hud_prefs,"$pref_showhud_text",
  "$pref_showhud_help",subby_ShowHUD);
 
-
-
 static tConfItem<bool> WRAP("WRAP_MENU",uMenu::wrap);
 
-
-
-#ifndef DEDICATED
-
 class gMemuItemConsole: uMenuItemStringWithHistory{
 public:
     gMemuItemConsole(uMenu *M,tString &c):
@@ -671,6 +679,8 @@ public:
 };
 
 void do_con(){
+    su_ClearKeys();
+        
     se_ChatState( ePlayerNetID::ChatFlags_Console, true );
     sr_con.SetHeight(20,false);
     se_SetShowScoresAuto(false);
@@ -692,7 +702,7 @@ void do_con(){
 
 void sg_ConsoleInput(){
 #ifndef DEDICATED
-    st_ToDo(&do_con);
+    st_ToDoOnce(&do_con);
 #endif
 }
 
@@ -837,13 +847,14 @@ public:
         }
         */
 #ifndef DEDICATED
+        uMenuItem::RenderBackground();
         if (!sr_glOut)
             return;
-        uMenuItem::RenderBackground();
         REAL r = rgb[0]/15.0;
         REAL g = rgb[1]/15.0;
         REAL b = rgb[2]/15.0;
         se_MakeColorValid(r, g, b, 1.0f);
+        RenderEnd();
         glColor3f(r, g, b);
         glRectf(.8,-.8,.98,-.98);
 #endif
@@ -1009,7 +1020,7 @@ void sg_PlayerMenu(int Player){
     (&camera_menu,
      "$player_camera_fov_text",
      "$player_camera_fov_help",
-     p->startFOV,30,120,5);
+     p->startFOV,30,160,5);
 
     uMenuItemSelection<eCamMode> cam_s
     (&camera_menu,
