@@ -128,7 +128,7 @@ static inline void arrow(REAL x,REAL y,REAL dy,REAL size){
 #endif
 }
 
-static bool repeat = false;
+static bool s_globalRepeat = false;
 
 #ifndef DEDICATED
 static bool disphelp=false;
@@ -146,6 +146,7 @@ static rNoAutoDisplayAtNewlineCallback su_noNewline( uMenu::MenuActive );
 
 void uMenu::OnEnter(){
 #ifndef DEDICATED
+    bool localRepeat = false;
     float nextrepeat = 0.0f;
     static const float repeatdelay = 0.2f;
     static const float repeatrateStart  = 0.2f;
@@ -208,12 +209,12 @@ void uMenu::OnEnter(){
                         // don't send keyup events when released.
                         break;
                     }
-                    repeat = true;
+                    localRepeat = s_globalRepeat = true;
                     memcpy( &tEventRepeat, &tEvent, sizeof( SDL_Event ) );
                     nextrepeat = tSysTimeFloat() + repeatdelay;
                     break;
                 case SDL_KEYUP:
-                    repeat = false;
+                    localRepeat = s_globalRepeat = false;
                     repeatrate = repeatrateStart;
                     break;
                 }
@@ -226,11 +227,11 @@ void uMenu::OnEnter(){
 
                 if ( tSysTimeFloat() - entertime > 1 )
                 {
-                    repeat = false;
+                    localRepeat = s_globalRepeat = false;
                 }
             }
 
-            if ( repeat && tSysTimeFloat() > nextrepeat )
+            if ( localRepeat && s_globalRepeat && tSysTimeFloat() > nextrepeat )
             {
                 this->HandleEvent( tEventRepeat );
                 nextrepeat = tSysTimeFloat() + repeatrate;
@@ -314,7 +315,7 @@ void uMenu::OnEnter(){
 
             Color(.6,.6,1,1);
             ::DisplayText(0,menuTop+text_height*titlefac
-                          ,text_width*titlefac,text_height*titlefac,
+                          ,text_width*titlefac*rTextField::AspectWidthMultiplier(),text_height*titlefac,
                           title,0);
 
             glDisable(GL_TEXTURE_2D);
@@ -341,7 +342,7 @@ void uMenu::OnEnter(){
                           .8*helpAlpha,
                           .8*helpAlpha);
 
-                rTextField c(-.95f,menuBot-.04f);
+                rTextField c(-.95f,menuBot-.04f,rCWIDTH_NORMAL*rTextField::AspectWidthMultiplier());
                 c.SetWidth(static_cast<int>((1.9f-items[selected]->SpaceRight())/c.GetCWidth()));
                 c << items[selected]->Help();
             }
@@ -359,7 +360,7 @@ void uMenu::OnEnter(){
 #endif
     }
 
-    repeat = false;
+    s_globalRepeat = false;
 
     uCallbackMenuLeave::MenuLeave();
     su_inMenu = false;
@@ -378,7 +379,7 @@ void uMenu::HandleEvent( SDL_Event event )
             switch (event.key.keysym.sym){
 
             case(SDLK_ESCAPE):
-                            repeat = false;
+                s_globalRepeat = false;
                 lastkey=tSysTimeFloat();
                 Exit();
                 break;
@@ -418,7 +419,7 @@ void uMenu::HandleEvent( SDL_Event event )
             case(SDLK_SPACE):
                         case(SDLK_KP_ENTER):
                             case(SDLK_RETURN):
-                                    repeat = false;
+                                    s_globalRepeat = false;
                 try
         {
                     su_inMenu = false;
@@ -449,7 +450,7 @@ void uMenu::HandleEvent( SDL_Event event )
 
                 su_inMenu = true;
 
-                repeat = false;
+                s_globalRepeat = false;
                 lastkey=tSysTimeFloat();
                 break;
 
@@ -588,6 +589,9 @@ void uMenuItem::DisplayText(REAL x,REAL y,const char *text,
 
         REAL tw = text_width;
         REAL th = text_height;
+        
+        //aspect ratio correction
+        tw *= (REAL(sr_screenHeight)/sr_screenWidth)*(4.0/3.0);
 
         
 #if 0
@@ -1174,7 +1178,7 @@ bool uMenu::IdleInput( bool processInput )
             switch (event.key.keysym.sym)
             {
             case(SDLK_ESCAPE):
-                repeat = false;
+                s_globalRepeat = false;
                 lastkey=tSysTimeFloat();
                 return true;
                 break;
@@ -1276,9 +1280,8 @@ bool uMenu::Message(const tOutput& message, const tOutput& interpretation, REAL
 
                 GenericBackground();
 
-                REAL w=16*3/640.0;
-                REAL h=32*3/480.0;
-
+                //16*3/640.0, 32*3/480.0
+                REAL w=0.1*(REAL(sr_screenHeight)/sr_screenWidth),h=0.2;
 
                 //REAL middle=-.6;
 
@@ -1293,7 +1296,8 @@ bool uMenu::Message(const tOutput& message, const tOutput& interpretation, REAL
                 Color(1,1,1);
                 DisplayText(0,.8,w,h, message);
 
-                w = 16/640.0;
+                //16/640.0
+                w = 1/30.0*(REAL(sr_screenHeight)/sr_screenWidth);
                 h = 32/480.0;
 
                 if (offset >= lines.size()) offset = lines.size() - 1;
