@@ -22,6 +22,11 @@
 // disable POD initialization behavior change warning in VisualC++ 2005
 #pragma warning ( disable: 4345 )
 
+// compatibility with later mingw versions
+#if defined(__GNUC__) && __GNUC__ >= 4
+#define HAVE_ISBLANK
+#endif
+
 // activate dirty OpenGL initialization option
 #ifndef DIRTY
 #define DIRTY
@@ -32,6 +37,12 @@
 #define WIN32
 #endif
 
+// exists in windows headers
+#define HAVE_ISBLANK
+
+// does not exist
+#undef HAVE_CLEARENV
+
 // defines for data directories in Windows
 #ifndef DEBUG
 #define USER_DATA_DIR  "${APPDATA}/Armagetron" 
